@@ -1,6 +1,7 @@
 
 import logging
 from os.path import dirname, join
+from unicodedata import normalize
 
 
 class Tagger(object):
@@ -35,10 +36,10 @@ class Tagger(object):
         if singular:
             word = singular[0]
         tags = [word]
-        if prev:
+        if prev_valid:
             # Combined with previous word.
             tags.append(prev + word)
-        if next:
+        if next_valid:
             # Combined with next word.
             tags.append(word + next)
         if prev_valid and next_valid:
@@ -55,25 +56,24 @@ class Tagger(object):
         best = None
         highscore = 0
         for tag in tags:
-            if len(tag) >= self.min_length:
-                score = self.scorer(tag)
-                logging.debug("Score for '%s': %s" % (tag, score))
-                if score > highscore:
-                    highscore = score
-                    best = tag
+            score = self.scorer(tag)
+            logging.debug("Score for '%s': %s" % (tag, score))
+            if score > highscore:
+                highscore = score
+                best = tag
         return best, highscore
 
-    def tags(self, text):
+    def tags(self, text, ascii=True):
         """
         Returns tags for the given text.
 
         Steps:
 
-        1) Go through every word in the text and if non-dictionary and
-           non-numeric, create up to 4 possible tags from it, the word
-           combined with the previous word, the next word, both previous
-           and next words together, and the word itself. Only use previous
-           and next words that aren't stopwords.
+        1) Go through every word in the text and if non-numeric, valid
+           length and non-dictionary, create up to 4 possible tags from
+           it, the word combined with the previous word, the next word,
+           both previous and next words together, and the word itself.
+           Only use previous and next words that aren't stopwords.
         2) Ignore all the possible tags from the word if any of
            them have already been added as tags, eg via the previous
            or next word iteration, or a duplicate word.
@@ -83,10 +83,13 @@ class Tagger(object):
 
         """
         logging.debug("Getting tags for: %s" % text)
-        # Initial list of alphanumeric words
         # Treat dashes and slashes as separators.
-        words = "".join([c for c in text.replace("-", " ").replace("/", " ")
-                         if c.isalnum() or c in "' "]).encode("utf-8").split()
+        text = unicode(text.replace("-", " ").replace("/", " "))
+        # Translate unicode chars to ascii.
+        if ascii:
+            text = normalize("NFKD", text).encode("ascii", "ignore")
+        # Initial list of alphanumeric words.
+        words = "".join([c for c in text if c.isalnum() or c in "' "]).split()
         # All tags mapped to scores.
         tags = {}
         for i, word in enumerate(words):
@@ -94,7 +97,8 @@ class Tagger(object):
             # Ignore numbers and numeric positions, eg: '1st' or '44th'.
             numeric = (word.lower().endswith(("st", "nd", "th")) and
                        word[:-2].isdigit()) or word.isdigit()
-            if not (numeric or word.lower() in self.dictionary):
+            too_short = len(word) < self.min_length
+            if not (numeric or too_short or word.lower() in self.dictionary):
                 possible = self.possible_for_index(words, i)
                 logging.debug("Possible tags for the word '%s': %s" %
                               (word, ", ".join(possible)))
